@@ -4,7 +4,7 @@ use strict;
 use warnings FATAL =>'all';
 
 # Initialize our version
-our $VERSION = '1.28';
+our $VERSION = '1.29';
 
 # Import what we need from the POE namespace
 use POE;
@@ -22,7 +22,7 @@ use Scalar::Util qw( reftype );
 use Carp;
 use vars qw($AUTOLOAD);
 
-our %COMMANDS = map { $_ => 1 } qw(                    
+our %COMMANDS = map { $_ => 1 } qw(
     commit
     rollback
     begin_work
@@ -48,11 +48,11 @@ sub AUTOLOAD {
     my $method = $AUTOLOAD;
     $method =~ s/.*:://;
     my $call = ( $method =~ s/^_// ) ? 1 : 0;
-    
+
     return unless $method =~ /[^A-Z]/;
-    
+
     $method = lc( $method );
-    
+
     croak "EasyDBI command $method does not exist"
         unless ( exists( $COMMANDS{ $method } ) );
 
@@ -102,7 +102,7 @@ sub new {
 
     # lowercase keys
     %opt = map { lc($_) => $opt{$_} } keys %opt;
-    
+
     my @valid = qw(
         dsn
         username
@@ -123,7 +123,7 @@ sub new {
         stopwatch
         query_logger
     );
-    
+
     # check the DSN
     # username/password/port other options
     # should be part of the DSN
@@ -160,7 +160,7 @@ sub new {
         # Set the default
         $opt{alias} = 'EasyDBI';
     }
-    
+
     # check for connect error event
     if (exists($opt{connect_error})) {
         if (_ARRAY($opt{connect_error})) {
@@ -168,14 +168,14 @@ sub new {
                 warn('connect_error must be an array reference that contains '
                 .'at least a session and event, ignoring');
                 delete $opt{connect_error};
-            }   
+            }
         } else {
             warn('connect_error must be an array reference that contains at '
             .'least a session and event, ignoring');
             delete $opt{connect_error};
         }
     }
-    
+
     # check for connect event
     if (exists($opt{connected})) {
         if (_ARRAY($opt{connected})) {
@@ -183,7 +183,7 @@ sub new {
                 warn('connected must be an array reference that contains '
                 .'at least a session and event, ignoring');
                 delete $opt{connected};
-            }   
+            }
         } else {
             warn('connected must be an array reference that contains at '
             .'least a session and event, ignoring');
@@ -205,7 +205,7 @@ sub new {
             delete $opt{stopwatch};
         }
     }
-    
+
     my $keep = { map { $_ => delete $opt{$_} } @valid };
 
     # Anything left over is unrecognized
@@ -213,7 +213,7 @@ sub new {
         croak('Unrecognized keys/options ('.join(',',(keys %opt))
             .') were present in new() call to POE::Component::EasyDBI!');
     }
-    
+
     my $self = bless($keep,ref $class || $class);
 
     # Create a new session for ourself
@@ -226,23 +226,23 @@ sub new {
                 '_stop'         =>  'stop',
                 'setup_wheel'   =>  'setup_wheel',
                 'sig_child'     =>  'sig_child',
-    
+
                 # child events
                 'child_error'   =>  'child_error',
                 'child_closed'  =>  'child_closed',
                 'child_STDOUT'  =>  'child_STDOUT',
                 'child_STDERR'  =>  'child_STDERR',
-   
+
                 # database events
                 (map { $_ => 'db_handler', uc($_) => 'db_handler' } keys %COMMANDS),
-                
-                # redefine 
+
+                # redefine
                 'combo'         =>  'combo_query',
                 'COMBO'         =>  'combo_query',
                 'shutdown'      =>  'shutdown_poco',
                 'SHUTDOWN'      =>  'shutdown_poco',
-   
-            
+
+
                 # Queue handling
                 'send_query'    =>  'send_query',
                 'check_queue'   =>  'check_queue',
@@ -296,17 +296,17 @@ sub new {
             },
         },
     ) or die 'Unable to create a new session!';
-    
+
     # save the session id
     $self->{ID} = $session->ID;
-    
+
     return wantarray ? ($self,$session) : $self;
 }
 
 # This subroutine handles shutdown signals
 sub shutdown_poco {
     my ($kernel, $heap) = @_[KERNEL,HEAP];
-    
+
     # Check for duplicate shutdown signals
     if ($heap->{shutdown}) {
         # Okay, let's see what's going on
@@ -338,7 +338,7 @@ sub shutdown_poco {
         # Go over our queue, and do some stuff
         foreach my $queue (@{$heap->{queue}}) {
             # Skip the special EXIT actions we might have put on the queue
-            if ($queue->{action} eq 'EXIT') { next }
+            if ($queue->{action} && $queue->{action} eq 'EXIT') { next }
 
             # Post a failure event to all the queries on the Queue
             # informing them that we have been shutdown...
@@ -355,7 +355,7 @@ sub shutdown_poco {
     } else {
         # Gracefully shut down...
         $heap->{shutdown} = 1;
-        
+
         # Put into the queue EXIT for the child
         $kernel->yield( 'send_query', {
                 action          =>  'EXIT',
@@ -373,14 +373,14 @@ sub combo_query {
     unless (_HASH($args)) {
         $args = { @_[ ARG0 .. $#_ ] };
     }
-    
+
     # Add some stuff to the args
     # defaults to sender, but can be specified
     unless (defined($args->{session})) {
         $args->{session} = $_[SENDER]->ID();
         DEBUG && print "setting session to $args->{session}\n";
     }
-    
+
     $args->{action} = $_[STATE];
 
     if (!exists($args->{event})) {
@@ -407,13 +407,13 @@ sub combo_query {
 
     foreach my $i ( 0 .. $#{ $args->{queries} } ) {
         my ($type, $arg) = %{ $args->{queries}->[ $i ] };
-        
+
         # Copy pass-through options
         for my $key ( keys %{ $args } ) {
             next if defined $arg->{$key} || $key eq 'queries';
             $arg->{$key} = $args->{$key};
         }
-        
+
         $arg->{event} = $handle;
         $arg->{__last} = ( $i == $#{ $args->{queries} } );
         $kernel->call( $_[SESSION] => $type => $arg );
@@ -440,14 +440,14 @@ sub db_handler {
     if ($args->{seperator}) {
         $args->{separator} = $args->{seperator};
     }
-    
+
     # Add some stuff to the args
     # defaults to sender, but can be specified
     unless (defined($args->{session})) {
         $args->{session} = $_[SENDER]->ID();
         DEBUG && print "setting session to $args->{session}\n";
     }
-    
+
     $args->{action} = $_[STATE];
 
     if (!exists($args->{event})) {
@@ -548,7 +548,7 @@ sub db_handler {
 
     # Increment the refcount for the session that is sending us this query
     $kernel->refcount_increment($args->{session}, 'EasyDBI');
-    
+
     if ($args->{session} ne $_[SENDER]->ID()) {
         $kernel->refcount_increment($_[SENDER]->ID(), 'EasyDBI');
         $args->{sendersession} = $_[SENDER]->ID();
@@ -556,14 +556,14 @@ sub db_handler {
 
     # Okay, fire off this query!
     $kernel->call($_[SESSION] => 'send_query' => $args);
-    
+
     return;
 }
 
 # This subroutine starts the process of sending a query
 sub send_query {
     my ($kernel, $heap, $args) = @_[KERNEL,HEAP,ARG0];
-    
+
     # Validate that we have something
     if (!defined($args) || !_HASH($args) ) {
         return;
@@ -577,23 +577,23 @@ sub send_query {
 
     # Send the query!
     $kernel->call($_[SESSION], 'check_queue');
-    
+
     return;
 }
 
 # This subroutine is the meat - sends queries to the subprocess
 sub check_queue {
     my ($kernel, $heap) = @_[KERNEL,HEAP];
-    
+
     # Check if the subprocess is currently active
     return unless (!$heap->{active});
 
     # Check if we have a query in the queue
     return unless (scalar(@{ $heap->{queue} }) > 0);
-    
+
     # shutting down?
     return unless ($heap->{shutdown} != 2);
-    
+
     if ($heap->{opts}{stopwatch}) {
         tie $heap->{queue}->[0]->{stopwatch}, 'Time::Stopwatch';
     }
@@ -626,20 +626,20 @@ sub print_queue {
 # This starts the EasyDBI
 sub start {
     my ($kernel, $heap) = @_[KERNEL,HEAP];
-    
+
     # Set up the alias for ourself
     $kernel->alias_set($heap->{alias}) if ($heap->{alias} ne '');
-    
+
     # Create the wheel
     $kernel->call( $_[SESSION] => 'setup_wheel' );
-    
+
     return;
 }
 
 # This sets up the WHEEL
 sub setup_wheel {
     my ($kernel, $heap) = @_[KERNEL,HEAP];
-    
+
     # Are we shutting down?
     if ($heap->{shutdown}) {
 #       if ($heap->{wheel}) {
@@ -671,7 +671,7 @@ sub setup_wheel {
 
 #    $kernel->sig_child( $heap->{wheel_pid} )
 #        if ( $heap->{wheel_pid} );
-    
+
     # Set up the SubProcess we communicate with
     $heap->{wheel} = POE::Wheel::Run->new(
         # What we will run in the separate process
@@ -702,7 +702,7 @@ sub setup_wheel {
     );
 
     $heap->{wheel_pid} = $heap->{wheel}->PID();
-    
+
     if ( $kernel->can( "sig_child" ) ) {
         $kernel->sig_child( $heap->{wheel_pid} => 'sig_child' );
     } else {
@@ -726,7 +726,7 @@ sub setup_wheel {
         # Check for queries
         $kernel->call($_[SESSION], 'check_queue');
     }
-    
+
     return;
 }
 
@@ -773,7 +773,7 @@ sub delete_query {
 # Handles child DIE'ing
 sub child_closed {
     my ($kernel, $heap) = @_[KERNEL,HEAP];
-    
+
     DEBUG && warn 'POE::Component::EasyDBI\'s Wheel closed';
     if ($heap->{shutdown}) {
 #       if ($heap->{wheel}) {
@@ -789,21 +789,21 @@ sub child_closed {
     # Create the wheel again
     delete $heap->{wheel};
     $kernel->call($_[SESSION], 'setup_wheel');
-    
+
     return;
 }
 
 # Handles child error
 sub child_error {
     my $heap = $_[HEAP];
-    
+
     # Emit warnings only if debug is on
     DEBUG && do {
         # Copied from POE::Wheel::Run manpage
         my ($operation, $errnum, $errstr) = @_[ARG0 .. ARG2];
         warn "POE::Component::EasyDBI got an $operation error $errnum from Subprocess: '$errstr' shutdown: $heap->{shutdown}\n";
     };
-    
+
     if ($heap->{shutdown}) {
         if ($heap->{wheel}) {
             $heap->{wheel}->kill();
@@ -816,7 +816,7 @@ sub child_error {
 # Handles child STDOUT output
 sub child_STDOUT {
     my ($self, $kernel, $heap, $data) = @_[OBJECT,KERNEL,HEAP,ARG0];
-    
+
     # Validate the argument
     unless ( _HASH($data) ) {
         warn "POE::Component::EasyDBI did not get a hash from the child ( $data )";
@@ -832,7 +832,6 @@ sub child_STDOUT {
     # Check for special DB messages with ID of 'DBI'
     if ($data->{id} eq 'DBI') {
         # Okay, we received a DBI error -> error in connection...
-
         if ($heap->{no_connect_failures}) {
             my $qc = {};
             if (defined($heap->{queue}->[0])) {
@@ -841,7 +840,9 @@ sub child_STDOUT {
             $qc->{error} = $data->{error};
             if (_ARRAY($heap->{opts}{connect_error})) {
                 $kernel->post(@{$heap->{opts}{connect_error}}, $qc);
-                delete $heap->{queue}[0]->{error};
+                if (_HASH($heap->{queue}[0])) {
+                    delete $heap->{queue}[0]->{error};
+                }
             } elsif ($qc->{session} && $qc->{event}) {
                 if ( reftype( $qc->{event} ) && reftype( $qc->{event} ) eq 'CODE' ) {
                     my $callback = delete $qc->{event};
@@ -856,7 +857,7 @@ sub child_STDOUT {
 #           print "DBI error: $data->{error}, retrying\n";
             return;
         }
-        
+
         # Shutdown ourself!
         $kernel->call($_[SESSION], 'shutdown', 'NOW');
 
@@ -877,7 +878,7 @@ sub child_STDOUT {
 
     my $query;
     my $refcount_decrement = 0;
-    
+
     if (exists($data->{chunked})) {
         # Get the query from the queue
         if (exists($data->{last_chunk})) {
@@ -901,30 +902,30 @@ sub child_STDOUT {
     # copy the query data, so we don't clobber the
     # stored data when using chunks
     #my $query_copy = { %{ $query } };
-    
+
     # marry data from the child to the data from the queue
     #%$query_copy = (%$query_copy, %$data);
-    
+
 #   my $query_copy = { (%$query, %$data) };
-    
+
 #   my $query_copy = $query;
 #   foreach (keys %$data) { $query_copy->{$_} = $data->{$_}; }
 
     my $query_copy = { %$query, %$data };
-    
+
     # undocumented
     $poe_kernel->call( $self->{query_logger} => _log => $query_copy )
         if ( $self->{query_logger} );
 
 #    my ($ses,$evt) = ("$query_copy->{session}", "$query_copy->{event}");
-    
+
 #    $kernel->call($ses => $evt => $query_copy);
-    
+
     #undef $query;
     #foreach my $k (keys %$data) {
     #   $query_copy->{$k} = $data->{$k};
     #}
-    
+
    if ( reftype( $query_copy->{event} ) && reftype( $query_copy->{event} ) eq 'CODE' ) {
        DEBUG && print "calling callback\n";
        my $callback = delete $query_copy->{event};
@@ -938,9 +939,9 @@ sub child_STDOUT {
     if ($refcount_decrement) {
         $heap->{active} = 0;
         $kernel->refcount_decrement($query_copy->{session}, 'EasyDBI');
-        
+
         if (defined($query_copy->{sendersession}) && $query_copy->{sendersession} ne $query_copy->{session}) {
-            $kernel->refcount_decrement($query_copy->{sendersession}, 'EasyDBI');    
+            $kernel->refcount_decrement($query_copy->{sendersession}, 'EasyDBI');
         }
 
         # Now, that we have got a result, check if we need to send another query
@@ -964,10 +965,10 @@ sub child_STDERR {
 
 sub sig_child {
     my ($kernel, $heap) = @_[KERNEL, HEAP];
-    
+
     delete $heap->{wheel_pid};
     $kernel->sig_handled();
-    
+
     if ( $heap->{shutdown} ) {
         # Remove our alias so we can be properly terminated
         $kernel->alias_remove($heap->{alias}) if ($heap->{alias} ne '');
@@ -1117,7 +1118,7 @@ This one is for mysql:
 
 This method will die on error or return success.
 
-Note the difference between dbname and database, that is dependant on the 
+Note the difference between dbname and database, that is dependant on the
 driver used, NOT EasyDBI
 
 NOTE: If the SubProcess could not connect to the DB, it will return an error,
@@ -1158,7 +1159,7 @@ This is the DB password EasyDBI will use when making the call to connect
 
 =item C<options>
 
-Pass a hash ref that normally would be after the $password param on a 
+Pass a hash ref that normally would be after the $password param on a
 DBI->connect call.
 
 =item C<max_retries>
@@ -1380,7 +1381,7 @@ or
         }
         $primary_key = $sth->{NAME}->[($primary_key-1)];
     }
-    
+
     for $i (0..$sth->{NUM_OF_FIELDS}-1) {
         $col{$sth->{NAME}->[$i]} = $i;
         push(@cols, $sth->{NAME}->[$i]);
@@ -1417,7 +1418,7 @@ or
 
 =item C<hasharray>
 
-    This query is for those queries where you will get more than one row 
+    This query is for those queries where you will get more than one row
     and column back.
 
     Internally, it does something like this:
@@ -1429,12 +1430,12 @@ or
         }
         $primary_key = $sth->{NAME}->[($primary_key-1)];
     }
-    
+
     for $i (0..$sth->{NUM_OF_FIELDS}-1) {
         $col{$sth->{NAME}->[$i]} = $i;
         push(@cols, $sth->{NAME}->[$i]);
     }
-    
+
     $sth = $dbh->prepare_cached($SQL);
     $sth->execute($PLACEHOLDERS);
     while (@row = $sth->fetch_array()) {
@@ -1699,8 +1700,8 @@ or
 
 =item C<func>
 
-    This is for calling $dbh->func(), when using a driver that supports it.  
-    
+    This is for calling $dbh->func(), when using a driver that supports it.
+
     Internally, it does this:
 
     return $dbh->func(@{$args});
@@ -1752,7 +1753,7 @@ or
 =item C<commit>
 
     This is for calling $dbh->commit(), if the driver supports it.
-    
+
     Internally, it does this:
 
     return $dbh->commit();
@@ -1777,7 +1778,7 @@ or
 =item C<rollback>
 
     This is for calling $dbh->rollback(), if the driver supports it.
-    
+
     Internally, it does this:
 
     return $dbh->rollback();
@@ -1802,7 +1803,7 @@ or
 =item C<begin_work>
 
     This is for calling $dbh->begin_work(), if the driver supports it.
-    
+
     Internally, it does this:
 
     return $dbh->begin_work();
@@ -1848,7 +1849,7 @@ or
 
     $kernel->call('EasyDBI', 'shutdown' => 'NOW');
 
-    ALL shutdown NOW's send kill 9 to thier children, beware of any 
+    ALL shutdown NOW's send kill 9 to thier children, beware of any
     transactions that you may be in. Your queries will revert if you are in
     transaction mode
 
@@ -1941,7 +1942,7 @@ prefix like _ in front of your custom keys.  For example:
         }
     );
 
-If I were to add an option 'filehandle' (for importing data from a file for 
+If I were to add an option 'filehandle' (for importing data from a file for
 instance) you don't want an upgrade to produce BAD results.
 
 =back
@@ -1992,7 +1993,7 @@ This will shutdown EasyDBI.
         username    => 'user',
         password    => 'pass',
     );
-    
+
     # Create our own session to communicate with EasyDBI
     POE::Session->create(
         inline_states => {
@@ -2007,7 +2008,7 @@ This will shutdown EasyDBI.
 
                 # 'single' is very different from the single query in SimpleDBI
                 # look at 'hash' to get those results
-                
+
                 # If you select more than one field, you will only get the last one
                 # unless you pass in a separator with what you want the fields seperated by
                 # to get null sperated values, pass in separator => "\0"
@@ -2026,7 +2027,7 @@ This will shutdown EasyDBI.
                         event => 'quote_handler',
                     }
                 );
-                
+
                 $_[KERNEL]->post('EasyDBI',
                     arrayhash => {
                         sql => 'SELECT user_id,user_login from users where logins = ?',
@@ -2034,16 +2035,16 @@ This will shutdown EasyDBI.
                         placeholders => [qw(53)],
                     }
                 );
-                
+
                 my $postback = $_[SESSION]->postback("arrayhash_handler",3,2,1);
-                
+
                 $_[KERNEL]->post('EasyDBI',
                     arrayhash => {
                         sql => 'SELECT user_id,user_login from users',
                         event => $postback,
                     }
                 );
-                
+
                 $_[KERNEL]->post('EasyDBI',
                     arrayarray => {
                         sql => 'SELECT * from locations',
@@ -2059,7 +2060,7 @@ This will shutdown EasyDBI.
                         primary_key => '1', # you can specify a primary key, or a number based on what column to use
                     }
                 );
-                
+
                 $_[KERNEL]->post('EasyDBI',
                     hasharray => {
                         sql => 'SELECT * from locations',
@@ -2067,7 +2068,7 @@ This will shutdown EasyDBI.
                         primary_key => "1",
                     }
                 );
-                
+
                 # you should use limit 1, it is NOT automaticly added
                 $_[KERNEL]->post('EasyDBI',
                     hash => {
@@ -2075,14 +2076,14 @@ This will shutdown EasyDBI.
                         event => 'hash_handler',
                     }
                 );
-                
+
                 $_[KERNEL]->post('EasyDBI',
                     array => {
                         sql => 'SELECT location_id from locations',
                         event => 'array_handler',
                     }
                 );
-                
+
                 $_[KERNEL]->post('EasyDBI',
                     keyvalhash => {
                         sql => 'SELECT location_id,location_name from locations',
@@ -2113,7 +2114,7 @@ This will shutdown EasyDBI.
                         # query that will return a value
                     },
                 );
-                
+
                 # 3 ways to shutdown
 
                 # This will let the existing queries finish, then shutdown
@@ -2205,7 +2206,7 @@ This will shutdown EasyDBI.
         #   error => Error occurred, check this first
         # }
     }
-    
+
     sub array_handler {
         # For array calls, we receive an array
         # $_[ARG0] = {
@@ -2218,7 +2219,7 @@ This will shutdown EasyDBI.
         #   error => Error occurred, check this first
         # }
     }
-    
+
     sub arrayarray_handler {
         # For array calls, we receive an array ref of array refs
         # $_[ARG0] = {
@@ -2229,7 +2230,7 @@ This will shutdown EasyDBI.
         #   error => Error occurred, check this first
         # }
     }
-    
+
     sub hash_handler {
         # For hash calls, we receive a hash
         # $_[ARG0] = {
@@ -2252,7 +2253,7 @@ This will shutdown EasyDBI.
         #   primary_key => primary key used
         # }
     }
-    
+
     sub insert_handle {
         # $_[ARG0] = {
         #   sql => The SQL you sent
